#######################################################################
### Code for simulations, examples, and figures
### Paciorek, C.J. 2013. Spatial models for point and areal data using
### Markov random fields on a fine grid. Electronic Journal of Statistics.
##########################################################################

# note that 'Bin' in the names in this file refers to the standard ICAR model ('binary' weights)

# a couple functions used in various places below
source('ejsAuxil.R')


### Fig. 2
library(spam)
library(fields)

# 75x75 grid
m1 <- m2 <- 75
m <- m1 * m2

# generate precision matrices

Qbin <- graphRead('bin', m1, m2)[[1]]
Qtps <- graphRead('tps', m1, m2)[[1]]

eBin <- eigen(Qbin)
eBin$values[m] <- eBin$values[m-1]
eTps <- eigen(Qtps)
eTps$values[(m-2):m] <- eTps$values[m-3]

# higher order CAR
locs2 <- expand.grid(1:m1,1:m2)
dd2 <- rdist(locs2)

rmax <- 3
Q <- matrix(0, m, m)
Q[dd2 <= rmax] <- -1
diag(Q) <- 0
diag(Q) <- -rowSums(Q)
Q <- as.spam(Q)
eHicar3 <- eigen(Q)

# distance-dependent CAR
rmax <- 5
psi <- -log(.05)/log(rmax)
Q <- -dd2^(-psi)
Q[dd2>rmax] <- 0
diag(Q) <- 0
diag(Q) <- -rowSums(Q)
Q <- as.spam(Q)
eDicar5 <- eigen(Q)

# Matern correlation
f.matern.euc <- function(dist, parm1, parm2){
  # first finding log values is about same speed as on original scale but may be more stable numerically
  const <- -lgamma(parm2)-(parm2-1)*log(2)
  tmp <- exp(const+parm2*log(2.0*sqrt(parm2)*dist/parm1)+log(besselK(2.0*sqrt(parm2)*dist/parm1, parm2)))
  tmp[tmp>1] <- 1
  tmp[dist == 0] <- 1
  return(tmp)
}

matCovHi <- f.matern.euc(dd2, m1, 2) 
matCovLo <- f.matern.euc(dd2, m1/25, 2) 
expCovHi <- f.matern.euc(dd2, m1, .5)  
expCovLo <- f.matern.euc(dd2, m1/25, .5)  
eMatCovHi <- eigen(matCovHi)
eMatCovLo <- eigen(matCovLo)
eExpCovHi <- eigen(expCovHi)
eExpCovLo <- eigen(expCovLo)

pdf('fig2.pdf', width = 8, height = 4) 
par(mfrow = c(1, 2), mgp = c(1.8, .7, 0), mai = c(.6, .6, .1, .1))
rg <- 2:1101
plot(rg, rev(log(eBin$val[5526]/eBin$val[4525:5624])), type = 'l', xlim = c(0, 1000), ylim = c(-5, 8), ylab = 'log relative eigenvalue', xlab = 'order of eigenvalue')
polygon(cbind(c(rg, rev(rg)), c(log(eMatCovLo$val[rg]/eMatCovLo$val[100]), log(eMatCovHi$val[rev(rg)]/eMatCovHi$val[100]))), border = 'NA', col = 'lightgoldenrod')
polygon(cbind(c(rg, rev(rg)), c(log(eExpCovLo$val[rg]/eExpCovLo$val[100]), log(eExpCovHi$val[rev(rg)]/eExpCovHi$val[100]))), border = 'NA', col = 'lightblue')
lines(rg, log(eMatCovHi$val[rg]/eMatCovHi$val[100]), col = 'darkorange')
lines(rg, log(eMatCovLo$val[rg]/eMatCovLo$val[100]), col = 'darkorange')
lines(rg, log(eExpCovHi$val[rg]/eExpCovHi$val[100]), col = 'blue')
lines(rg, log(eExpCovLo$val[rg]/eExpCovLo$val[100]), col = 'blue')
lines(rg, rev(log(eTps$val[5526]/eTps$val[4525:5624])), col = 'red')
lines(rg, rev(log(eBin$val[5526]/eBin$val[4525:5624])), col = 'black')
box()
legend(400, 8, col = c('black', 'red', 'blue', 'darkorange'), lty = rep(1, 4), legend = c('ICAR', 'TPS-MRF', 'exponential', 'Matern, v = 2'),  bty = 'n')
text('a', 980, 8)

# show CAR competitors here
rg <- 1:1100
wh <- 4525:5624
plot(rg, rev(log(eBin$val[5526]/eBin$val[wh])), type = 'l', xlim = c(0, 1000), ylim = c(-5, 8), ylab = 'log relative eigenvalue', xlab = 'order of eigenvalue')
lines(rg, rev(log(eTps$val[5526]/eTps$val[wh])), col = 'red')
lines(rg, rev(log(eHicar3$val[5526]/eHicar3$val[wh])), col = 'purple')
lines(rg, rev(log(eDicar5$val[5526]/eDicar5$val[wh])), col = 'green')
text('b', 980, 8)
legend(500, 8, col = c('black', 'red', 'purple', 'green'), lty = rep(1, 6), legend = c('ICAR', 'TPS-MRF', 'HICAR', 'DICAR'),  bty = 'n')

dev.off()  # end pdf


### Figs. 3-4

focal <- 37*75+37

out <- list()
lambdaVals <- exp(c(2, 4))

locs2 <- expand.grid(1:m1, 1:m2)
dd2 <- rdist(locs2)
matCovMed <- f.matern.euc(dd2, m1/10, 2) 
expCovMed <- f.matern.euc(dd2, m1/10, .5) 

# calculate equivalent kernels
for(j in 1:2){ # loop over two values of lambda
  out[[j]] <- matrix(NA, 6, m)

  lambda <- lambdaVals[j]

  S <- lambda*Qbin
  diag(S) <- diag(S)+1
  out[[j]][1, ] <- solve(S)[focal, ]
  
  S <- lambda*Qtps
  diag(S) <- diag(S)+1
  out[[j]][2, ] <- solve(S)[focal, ]

  # higher-order neighborhood structure
  rmax <- 3
  Q <- matrix(0, m, m)
  Q[dd2 <= rmax] <- -1
  diag(Q) <- 0
  diag(Q) <- -rowSums(Q)
  S <- lambda*as.spam(Q)
  diag(S) <- diag(S)+1
  out[[j]][3, ] <- solve(S)[focal, ]

  # distance dependent CAR
  rmax <- 5
  psi <- -log(.05)/log(rmax)
  Q <- -dd2^(-psi)
  Q[dd2>rmax] <- 0
  diag(Q) <- 0
  diag(Q) <- -rowSums(Q)
  S <- lambda*as.spam(Q)
  diag(S) <- diag(S)+1
  out[[j]][4, ] <- solve(S)[focal, ]
  
  tmp <- matCovMed
  diag(tmp) <- diag(tmp)+lambda
  out[[j]][5, ] <- solve(tmp, matCovMed)[, focal]
  
  tmp <- expCovMed
  diag(tmp) <- diag(tmp)+lambda
  out[[j]][6, ] <- solve(tmp, expCovMed)[, focal]
}
  
### plotting

hi <- c(.06, .018)
lo <- rep(-.001, 2)
focal <- 37*75+37

pdf('fig3.pdf', width = 8, height = 6.5) 
par(mfrow = c(2, 3), mai = c(.25, .1, .1, .05), mgp = c(1.85, .8, 0), omi = c(.2, .35, 0, 0), cex.axis = 1.1)
rg <- 25
j <- 2 # (exp(4))
plot((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)], ylim = c(lo[j], hi[j]), xlab = '', ylab = '', type = 'l')
abline(h = 0, col = 'lightgray', lwd = 1.5)
lines((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)])
lines((-rg):rg, out[[j]][2, (focal-rg):(focal+rg)], col = 'red')
text(-24.5, .018, '(a)', cex = 1.2)
legend(2, .017, lty = 1, col = c('black', 'red'), legend = c('ICAR', 'TPS-MRF'),  bty = 'n', cex = 1.1)
title(ylab = 'weight', outer = TRUE, line = 1.25, cex.lab = 1.3)

plot((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)], ylim = c(lo[j], hi[j]), xlab = '', ylab = '', type = 'l', yaxt = 'n')
abline(h = 0, col = 'lightgray', lwd = 1.5)
lines((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)])
lines((-rg):rg, out[[j]][3, (focal-rg):(focal+rg)], col = 'purple') # HICAR
lines((-rg):rg, out[[j]][4, (focal-rg):(focal+rg)], col = 'green') # DICAR
text(-24.5, .018, '(b)', cex = 1.2)
legend(5, .017, lty = 1, col = c('black', 'purple', 'green'), legend = c('ICAR', 'HICAR', 'DICAR'),  bty = 'n', cex = 1.1)

plot((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)], ylim = c(lo[j], hi[j]), xlab = '', ylab = '', type = 'l', yaxt = 'n')
abline(h = 0, col = 'lightgray', lwd = 1.5)
lines((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)])
lines((-rg):rg, out[[j]][2, (focal-rg):(focal+rg)], col = 'red')
lines((-rg):rg, out[[j]][6, (focal-rg):(focal+rg)], col = 'blue')
lines((-rg):rg, out[[j]][5, (focal-rg):(focal+rg)], col = 'orange')
text(-24.5, .018, '(c)', cex = 1.2)
legend(2, .017, lty = 1, col = c('black', 'red', 'blue', 'orange'), legend = c('ICAR', 'TPS-MRF', 'exponential', 'Matern, v=2'),  bty = 'n', cex = 1.1)
title(xlab = 'grid cell position relative to focal cell', outer = TRUE, line = 0.3, cex.lab = 1.3)

rg <- 12
j <- 1 # (exp(2))
plot((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)], ylim = c(lo[j], hi[j]), xlab = '', ylab = '', type = 'l')
abline(h = 0, col = 'lightgray', lwd = 1.5)
lines((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)])
lines((-rg):rg, out[[j]][2, (focal-rg):(focal+rg)], col = 'red')
text(-(rg-.25), .06, '(d)', cex = 1.2)


plot((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)], ylim = c(lo[j], hi[j]), xlab = '', ylab = '', type = 'l', yaxt = 'n')
abline(h = 0, col = 'lightgray', lwd = 1.5)
lines((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)])
lines((-rg):rg, out[[j]][3, (focal-rg):(focal+rg)], col = 'purple')
lines((-rg):rg, out[[j]][4, (focal-rg):(focal+rg)], col = 'green')
text(-(rg-.25), .06, '(e)', cex = 1.2)

plot((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)], ylim = c(lo[j], hi[j]), xlab = '', ylab = '', type = 'l', yaxt = 'n')
abline(h = 0, col = 'lightgray', lwd = 1.5)
lines((-rg):rg, out[[j]][1, (focal-rg):(focal+rg)])
lines((-rg):rg, out[[j]][2, (focal-rg):(focal+rg)], col = 'red')
lines((-rg):rg, out[[j]][6, (focal-rg):(focal+rg)], col = 'blue')
lines((-rg):rg, out[[j]][5, (focal-rg):(focal+rg)], col = 'orange')
text(-(rg-.25), .06, '(f)', cex = 1.2)

dev.off() # end pdf


j <- 2 # lambda=exp(4)

pdf('fig4.pdf', width = 10.5, height = 2.35)
par(mfrow = c(1, 4), mai = c(.1, .1, .3, .4), omi = c(0, 0, 0, .3))
image(1:m1, 1:m2, matrix(out[[j]][1, ], m1, m2), col = temp.colors(18*2+1)[19:(18*2+1)], xlim = c(37-10, 37+10), ylim = c(38-10, 38+10), xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', zlim = c(-0.0005, .0185), main = '(a) ICAR', bty = 'n')
image(1:m1, 1:m2, matrix(out[[j]][2, ], m1, m2), col = temp.colors(18*2+1)[19:(18*2+1)], xlim = c(37-10, 37+10), ylim = c(38-10, 38+10), xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', zlim = c(-0.0005, .0185), main = '(b) TPS-MRF', bty = 'n')
image(1:m1, 1:m2, matrix(out[[j]][6, ], m1, m2), col = temp.colors(18*2+1)[19:(18*2+1)], xlim = c(37-10, 37+10), ylim = c(38-10, 38+10), xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', zlim = c(-0.0005, .0185), main = '(c) exponential', bty = 'n')
image.plot(1:m1, 1:m2, matrix(out[[j]][5, ], m1, m2), col = temp.colors(18*2+1)[19:(18*2+1)], xlim = c(37-10, 37+10), ylim = c(38-10, 38+10), xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', zlim = c(-0.0005, .0185), main = '(d) Matern, v=2', bty = 'n', legend.width = 3)
dev.off() # end pdf


### oracle analysis for point-reference data for Figs. 5a and 6a
library(fields)
library(spam)


m1 <- m2 <- 100
m <- m1 * m2

# generate precision matrices
Qbin <- graphRead('bin', m1, m2)[[1]]
Qtps <- graphRead('tps', m1, m2)[[1]]

locs <- as.matrix(expand.grid(seq(0, 1, len = m1), seq(0, 1, len = m2)))
dd11 <- rdist(locs)

tol <- .001

sig2 <- 1
tau2vals <- (c(.05, .15, .45, 1.35))^2 
rhovals <- c(.005, .02, .08, .32, 1.28, 2.56)
nuvals <- c(.5, 2)

nReplicates <- 10 # to average over variability in sampling locations 

sseGP <- varGP <- biasVarGP <- array(NA, c(nReplicates, 2, length(rhovals), length(tau2vals)))
sseOracle <- varOracle <- biasVarOracle <- array(NA, c(nReplicates, 2, 2, length(rhovals), length(tau2vals)))
lamOracle <- array(NA, c(nReplicates, 2, 2, length(rhovals), length(tau2vals)))

# functions for calculating SSE, bias, variance for CAR and TPS

sseFunBin <- function(logL){
  lambda <- exp(logL)
  Sbin <- lambda*Qbin
  diag(Sbin) <- diag(Sbin)+PtP
  Mbin <- solve(Sbin, Pt)
  term1bin <- sum(rowSums(Mbin*Mbin))
  MtMbin <- t(Mbin)%*%Mbin
  PtMtMbin <- Pts%*%MtMbin
  lastTermBin <- sum(PtMtMbin*tPSig)
  termsBin <- -2*sum(Mbin*tPSig)+m1*m2*sig2+lastTermBin
  return(termsBin+tau2*term1bin)
}

biasVarFunBin <- function(logL){
  lambda <- exp(logL)
  Sbin <- lambda*Qbin
  diag(Sbin) <- diag(Sbin)+PtP
  Mbin <- solve(Sbin, Pt)
  term1bin <- sum(rowSums(Mbin*Mbin))
  MtMbin <- t(Mbin)%*%Mbin
  PtMtMbin <- Pts%*%MtMbin
  lastTermBin <- sum(PtMtMbin*tPSig)
  termsBin <- -2*sum(Mbin*tPSig)+m1*m2*sig2+lastTermBin
  return(c(tau2*term1bin/(termsBin+tau2*term1bin), tau2*term1bin))
}

sseFunTps <- function(logL){
  lambda <- exp(logL)
  Stps <- lambda*Qtps
  diag(Stps) <- diag(Stps)+PtP
  Mtps <- solve(Stps, Pt)
  term1tps <- sum(rowSums(Mtps*Mtps))
  MtMtps <- t(Mtps)%*%Mtps
  PtMtMtps <- Pts%*%MtMtps
  lastTermTps <- sum(PtMtMtps*tPSig)
  termsTps <- -2*sum(Mtps*tPSig)+m1*m2*sig2+lastTermTps
  return(termsTps+tau2*term1tps)
}

biasVarFunTps <- function(logL){
  lambda <- exp(logL)
  Stps <- lambda*Qtps
  diag(Stps) <- diag(Stps)+PtP
  Mtps <- solve(Stps, Pt)
  term1tps <- sum(rowSums(Mtps*Mtps))
  MtMtps <- t(Mtps)%*%Mtps
  PtMtMtps <- Pts%*%MtMtps
  lastTermTps <- sum(PtMtMtps*tPSig)
  termsTps <- -2*sum(Mtps*tPSig)+m1*m2*sig2+lastTermTps
  return(c(tau2*term1tps/(termsTps+tau2*term1tps), tau2*term1tps))
}

for(pattern in c('Unif', 'Cluster')){
  for(n in c(100, 1000)) {
    set.seed(0) # same RNG for each sample size, cluster/uniform pattern
    for(i in 1:nReplicates){
      
      if(pattern == 'Unif'){
        sites <- cbind(runif(n), runif(n))
      } else{ # 'cluster'
        sites <- pcpGen(n)$locs
        
      }
      dd21 <- rdist(sites, locs)
      filledCells  <- apply(dd21, 1, which.min)
      
                                        # these objects are for MRF calcs
      PtP <- rep(0, m)
      PtP[sort(unique(filledCells))] <- table(filledCells)
      
      Pt <- matrix(0, nr = m, nc = n)
      Pt[cbind(filledCells, 1:n)] <- 1
      Pts <- as.spam(Pt)
      
                                        # this is for GP
      dd22 <- rdist(sites,  sites)
      
### GP oracle results
      
      for(k in 1:2){
        nu <- nuvals[k]
        for(l in 1:length(rhovals)){
          rho <- rhovals[l]
          
          RPt <- f.matern.euc(rbind(t(dd21), dd22), rho, nu)  # (m+n) x n
          PRPt <- RPt[(m+1):(n+m), ]  # n x n
          
          for(mm in 1:length(tau2vals)){
            tau2 <- tau2vals[mm]
            lambda <- tau2/sig2
            
            C <- PRPt
            diag(C) <- diag(C)+lambda
            term <- solve(C, PRPt)
            M <- (1/lambda)*(RPt-RPt%*%term)[1:m, ]  # m x n
            
            MtM <- t(M)%*%M
            
            term1 <- tau2*sum(diag(MtM))
            term2 <- sig2*sum(MtM * RPt[(m+1):(m+n), ]) - 2*sig2*sum(M * RPt[1:m, ]) + sig2*m
            sseGP[i, k, l, mm] <- term1+term2
            varGP[i, k, l, mm] <- term1
            biasVarGP[i, k, l, mm] <- term1/(term1+term2)
            
            print(c(i, k, l, mm, date()))
          }
        }
      }
      rm(PRPt, RPt, C, M, MtM, term)
      
### MRF oracle results
      
      for(k in 1:2){
        nu <- nuvals[k]
        for(l in 1:length(rhovals)){
          rho <- rhovals[l]
          
          tPSig <- sig2*t(f.matern.euc(dd11[filledCells, ], rho, nu))  
          for(mm in 1:length(tau2vals)){
            tau2 <- tau2vals[mm]
            
            fitBin4 <- nlminb(1, sseFunBin, lower = -15, upper = 15)
            fitTps4 <- nlminb(1, sseFunTps, lower = -15, upper = 15)
            
            lamOracle[i, 1, k, l, mm] <- fitBin4$par
            sseOracle[i, 1, k, l, mm] <- fitBin4$objective
            
            lamOracle[i, 2, k, l, mm] <- fitTps4$par
            sseOracle[i, 2, k, l, mm] <- fitTps4$objective
            
            tmp <- biasVarFunBin(fitBin4$par)
            varOracle[i, 1, k, l, mm] <- tmp[2]
            biasVarOracle[i, 1, k, l, mm] <- tmp[1]
            
            tmp <- biasVarFunTps(fitTps4$par)
            varOracle[i, 2, k, l, mm] <- tmp[2]
            biasVarOracle[i, 2, k, l, mm] <- tmp[1]
            
            print(c(i, k, l, mm, date()))
          }
        }
      }
      rm(tPSig)
    }
    
    # unique output objects for each pattern and n
    assign(paste('sseOracle', pattern, n, sep = ''), sseOracle)
    assign(paste('sseGP', pattern, n, sep = ''), sseGP)
  }
}


### Figs. 5a and 6a

for(n in c(100, 1000)){ 
  sseOracleUnif <- get(paste('sseOracleUnif', n, sep = ''))
  sseGPUnif <- get(paste('sseGPUnif', n, sep = ''))
  sseOracleCluster <- get(paste('sseOracleCluster', n, sep = ''))
  sseGPCluster <- get(paste('sseGPCluster', n, sep = ''))
  
  if(n == 100) pdf('fig5a.pdf',height = 4,width = 8) else pdf('fig6a.pdf', height = 4, width = 8)
  par(mfrow = c(2, 4), mai = rep(.05,  4), mgp = c(1.8, .7, 0), omi = c(.4, .6, .45, .4))
  for(k in 1:2){  # loop over nu=1,nu=2
    for(mm in 1:length(tau2vals)){ 
      plotFun(sseOracleUnif, sseGPUnif, sseOracleCluster, sseGPCluster, k, mm, type = 'sparse')
      if(k == 1 && mm == 1)
        mtext(expression(nu == 0.5), at = .77, side = 2, outer = TRUE, cex = 1, line = 3)
      if(k == 2 && mm == 1){
        mtext(expression(nu == 2), at = .27, side = 2, outer = TRUE, cex = 1, line = 3)
      }
      if(k == 1 && mm == 1){
        mtext(expression(tau^2 == 0.05^2), at = .13, side = 3, outer = TRUE, cex = 1)
      }
      if(k == 1 && mm == 2){
        mtext(expression(tau^2 == 0.15^2), at = .37, side = 3, outer = TRUE, cex=1)
      }
      if(k == 1 && mm == 3){
        mtext(expression(tau^2 == 0.45^2), at = .63, side = 3, outer = TRUE, cex = 1)
      }
      if(k == 1 && mm == 4){
        mtext(expression(tau^2 == 1.35^2), at = .87, side = 3, outer = TRUE, cex = 1)
      }
      if(k == 2 && mm == 4)
        mtext('TPS-MRF SSE', outer = TRUE, side = 4, line = 1.7, col = "red", cex=.8, at = .5) 
      if(k == 1 && mm == 1)
        title(main = "(a) oracle results", outer = TRUE, line = 2, cex.main = 1.6, font.main = 1)
    }
  }
  dev.off()  # end pdf
}

### simulations for point-referenced data for Figs. 5b and 6b
  
library(splancs)
library(fields)
library(spam)


m1 <- m2 <- 100
m <- m1 * m2

# generate precision matrices
Qbin <- graphRead('bin', m1, m2)[[1]]
Qtps <- graphRead('tps', m1, m2)[[1]]

cover <- TRUE

nReps <- 100

tau2vals <- (c(.05,.15,.45,1.35))^2
rhovals <- c(.005, .02, .08, .32, 1.28, 2.56)

for(n in c(100, 1000)) {
  
  
  tau2HatBin <- tau2HatTps <- lambdaHatBin <- lambdaHatTps <- array(NA, c(96, nReps))
  gSave <- gHatBin <- gHatTps <- array(NA,c(96, nReps, m))
  sseBin <- sseTps <- mseBin <- mseTps <- array(NA, c(96, nReps, 4))
  
  if(cover){
    gSEBin <- gSETps <- gHatBin
    covBin <- covTps <- covBinPr <- covTpsPr <- array(NA,c(96, nReps))
  }
  lambdaHatGP <- lambdaHatBin
  rhoHatGP <- tau2HatGP <- tau2HatBin
  sseGP <- mseGP <- mseBin
  gHatGP <- gHatBin
  betaHatGP <- array(NA,c(96, nReps, 3))
  
  gHatGPorac <- gHatBin
  sseGPorac <- mseGPorac <- mseBin

  cnt <- 1
  # loop over different settings - this would take a very long time - in reality done via multiple jobs
  for(pattern in c('unif', 'cluster')){
    for(tauI in 1:length(tau2vals)){ # would be better to have this as the inner loop as Sigma does not change when tau2 changes, but ordering in the output objects matters (for plotting) so be careful
      tau2 <- tau2vals[tauI]
      for(rhoI in 1:6){
        rho <- rhovals[rhoI]
        for(nuI in 1:2){
          
          if(nuI==1){ # careful here - for historical reasons, nu=2 is nuI=1 and nu=0.5 is nuI=2
            nusize <- 'large'
          } else{
            nusize <- 'small'
          }
          
          set.seed(0)
          
          tol <- .001
          
          sig2 <- 1
          mu <- 0
          
          cTps <- 3
          cBin <- 1
          
          if(nusize == 'small'){
            nu <- .5; wrongNu <- 2
          } else{
            nu <- 2; wrongNu <- .5
          }
          
          gridLocs <- expand.grid(x = seq(0, 1, len=m1),  y = seq(0, 1, len=m2))
          
          dd <- Sigma <- matrix(0, m+n, m+n)
          dd[1:m, 1:m] <- rdist(gridLocs)
          Sigma[1:m, 1:m] <- sig2*f.matern.euc(dd[1:m, 1:m], rho, nu)
          idenM <- as.spam(diag(rep(1, m)))
          
          for(it in 1:nReps){
            
            if(pattern == 'unif'){
              locs <- cbind(runif(n), runif(n))
            } else{
              locs <- pcpGen(n)$locs
            }
            
            dists <- rdist(locs, gridLocs)
            filledCells <- apply(dists, 1, which.min)
            PtP <- rep(0, m)
            PtP[sort(unique(filledCells))] <- table(filledCells)
            Pt <- matrix(0, nr=m, nc=n)
            Pt[cbind(filledCells, 1:n)] <- 1
            Pt <- as.spam(Pt)
            
            distsP <- rdist(locs)
            
            tmp <- Qbin
            diag(tmp) <- diag(tmp)+PtP
            chBin <- chol.spam(tmp)
            
            tmp <- Qtps
            diag(tmp) <- diag(tmp)+PtP
            chTps <- chol.spam(tmp)
            
            poly <- locs[chull(locs), ]
            inhull <- inout(gridLocs, poly)
            
            close <- apply(dists, 2, min)<rho/2
            
            X <- as.matrix(cbind(rep(1, m), gridLocs))
            XP <- X[filledCells, ]
            XPt <- t(XP)
            
            
            Sigma[(m+1):(n+m), (m+1):(n+m)] <- sig2*f.matern.euc(distsP, rho, nu)
            Sigma[(m+1):(n+m), 1:m] <- sig2*f.matern.euc(dists, rho, nu)
            Sigma[1:m, (m+1):(n+m)] <- t(Sigma[(m+1):(n+m), 1:m])
            
            dd[(m+1):(n+m), (m+1):(n+m)] <- distsP
            dd[1:m, (m+1):(n+m)] <- t(dists)
            dd[(m+1):(n+m), 1:m] <- dists
            rm(dists)
            
            if(rho < 2){
              L <- t(chol(Sigma))  # 50s on cluster (or 3min)
              gfull <- mu+L%*%rnorm(m+n)
              g <- gfull[1:m]
              gPoint <- gfull[(m+1):(m+n)]
              rm(L)
            } else{
              U <- chol(Sigma[1:m,  1:m])
              z <- rnorm(m + n)
              g <- mu + crossprod(U, z[1:m])
              C21 <- Sigma[(m+1):(n+m), 1:m]
              gPoint <- mu + C21 %*% backsolve(U, z[1:m])
              predCov <- Sigma[(m+1):(m+n), (m+1):(m+n)] - C21 %*% backsolve(U, backsolve(U, t(C21), transpose = TRUE))
              predCov <- 0.5 * (predCov + t(predCov))
              e <- eigen(predCov)
              e$values[e$values < 0] <- 0
              gPoint <- gPoint + e$vectors %*% sqrt(e$values)*z[(m+1):(m+n)]
            }
            y <- rnorm(n, gPoint, sqrt(tau2))  
            yPr <- rnorm(m, g, sqrt(tau2))
            gSave[cnt,  it, ] <- g
            
            # fitting and prediction
            
            yty <- sum(y^2)
            Pty <- Pt%*%y
            
            # bin fitting
            negLogLik <- function(logL){
              lam <- exp(logL)
              Qplus <- lam*Qbin
              diag(Qplus) <- diag(Qplus)+PtP
              chBin <- update.spam.chol.NgPeyton(chBin, Qplus)
              
              tmp <- forwardsolve(chBin, Pty)
              return(-0.5*(m-cBin)*logL+0.5*(n-cBin)*log(yty-sum(tmp^2))+sum(log(diag(chBin))))
            }
            
            
            fit4 <- nlminb(1, negLogLik, lower = -15, upper = 15)
            lambdaHat <- exp(fit4$par)
            
            Qplus <- lambdaHat*Qbin
            diag(Qplus) <- diag(Qplus)+PtP
            chBin <- update.spam.chol.NgPeyton(chBin, Qplus)
            
            tmp <- forwardsolve(chBin, Pty)
            tau2Hat <- (yty-sum(tmp^2))/(n-cBin)
            
            kappaHatBin <- lambdaHat/tau2Hat
            
            gHat <- backsolve(chBin, forwardsolve(chBin, Pty))
            sseBin[cnt,  it, ] <- c(sum((g-gHat)^2), sum((g-gHat)[inhull == TRUE]^2), sum((g-gHat)[close == TRUE]^2), sum((g-gHat)[close == FALSE & inhull == TRUE]^2))
            mseBin[cnt,  it, ] <- c(mean((g-gHat)^2), mean((g-gHat)[inhull == TRUE]^2), mean((g-gHat)[close == TRUE]^2), mean((g-gHat)[close == FALSE & inhull == TRUE]^2))
            
            if(cover){
              tmp <- forwardsolve(chBin, idenM)
              tmp2 <- colSums(tmp*tmp)
              gSE <- sqrt(tau2Hat*tmp2) # faster
              rm(tmp)
              covBin[cnt,  it] <- mean(g<gHat+1.96*gSE & g>gHat-1.96*gSE)
              gSEpr <- sqrt(tau2Hat*(tmp2+1))
              covBinPr[cnt,  it] <- mean(yPr<gHat+1.96*gSEpr & yPr>gHat-1.96*gSEpr)
              gSEBin[cnt,  it, ] <- gSE
            }
            
            gHatBin[cnt,  it, ] <- gHat
            lambdaHatBin[cnt, it] <- lambdaHat
            tau2HatBin[cnt, it] <- tau2Hat
            
            # tps fitting
            negLogLik <- function(logL){
              lam <- exp(logL)
              
              Qplus <- lam*Qtps
              diag(Qplus) <- diag(Qplus)+PtP
              chTps <- update.spam.chol.NgPeyton(chTps, Qplus)
              
              tmp <- forwardsolve(chTps, Pty)
              return(-0.5*(m-cTps)*logL+0.5*(n-cTps)*log(yty-sum(tmp^2))+sum(log(diag(chTps))))
            }
            
            fit4 <- nlminb(1, negLogLik, lower = -15, upper = 15)
            
            lambdaHat <- exp(fit4$par)
            
            Qplus <- lambdaHat*Qtps
            diag(Qplus) <- diag(Qplus)+PtP
            chTps <- update.spam.chol.NgPeyton(chTps, Qplus)
            
            tmp <- forwardsolve(chTps, Pty)
            tau2Hat <- (yty-sum(tmp^2))/(n-cTps)
            
            kappaHatTps <- lambdaHat/tau2Hat
            
            gHat <- backsolve(chTps, forwardsolve(chTps, Pty))
            sseTps[cnt, it,] <- c(sum((g-gHat)^2), sum((g-gHat)[inhull == TRUE]^2), sum((g-gHat)[close == TRUE]^2), sum((g-gHat)[close == FALSE & inhull == TRUE]^2))
            mseTps[cnt, it, ] <- c(mean((g-gHat)^2), mean((g-gHat)[inhull == TRUE]^2), mean((g-gHat)[close == TRUE]^2), mean((g-gHat)[close == FALSE & inhull == TRUE]^2))
            
            if(cover){
              tmp <- forwardsolve(chTps, idenM)
              tmp2 <- colSums(tmp*tmp)
              gSE <- sqrt(tau2Hat*tmp2) # faster
              rm(tmp)
              covTps[cnt, it] <- mean(g<gHat+1.96*gSE & g>gHat-1.96*gSE)
              gSEpr <- sqrt(tau2Hat*(tmp2+1))
              covTpsPr[cnt, it] <- mean(yPr<gHat+1.96*gSEpr & yPr>gHat-1.96*gSEpr)
              gSETps[cnt, it, ] <- gSE
            }
            
            gHatTps[cnt, it, ] <- gHat
            lambdaHatTps[cnt, it] <- lambdaHat
            tau2HatTps[cnt, it] <- tau2Hat
            
            # GP with known params; beta assumed 0; dividing by sig2 since Sigma includes true sig2 (albeit equal to 1)
            R12 <- (1/sig2) * Sigma[1:m, (m+1):(m+n)]
            lambda <- tau2 / sig2 
            
            C <- (1/sig2) * Sigma[(m+1):(m+n), (m+1):(m+n)]
            diag(C) <- diag(C) + lambda
            gHat <- R12 %*% solve(C, y)
            
            sseGPorac[cnt, it, ] <- c(sum((g-gHat)^2), sum((g-gHat)[inhull == TRUE]^2), sum((g-gHat)[close == TRUE]^2), sum((g-gHat)[close == FALSE & inhull == TRUE]^2))
            mseGPorac[cnt, it, ] <- c(mean((g-gHat)^2), mean((g-gHat)[inhull == TRUE]^2), mean((g-gHat)[close == TRUE]^2), mean((g-gHat)[close == FALSE & inhull == TRUE]^2))
            gHatGPorac[cnt, it, ] <- gHat
            
            XP.gp <- as.matrix(cbind(rep(1, n), locs))
            
            
            # GP w/ estimated params
            negLogLik <- function(par){
              lam <- exp(par[1])
              rhoVal <- exp(par[2])
              
              C <- (1/lam)*f.matern.euc(distsP, rhoVal, nu)
              diag(C) <- diag(C)+1
              Lt <- chol(C)
              
              CinvY <- backsolve(Lt, backsolve(Lt, y, trans=TRUE))
              tmp <- backsolve(Lt, XP.gp, trans = TRUE)
              XtCinvX <- t(tmp)%*%tmp
              vec <- y-XP%*%solve(XtCinvX, t(XP.gp)%*%CinvY)
              CinvVec <- backsolve(Lt, vec, trans = TRUE)
              return(sum(log(diag(Lt)))+(n/2)*log(sum(CinvVec^2)))
            }
            
            fit4 <- nlminb(c(log(.5), log(.25)), negLogLik, lower = c(-15, log(.001)), upper = c(15, log(10)))
            
            lambdaHat <- exp(fit4$par[1])
            rhoHat <- exp(fit4$par[2])
            
            C <- (1/lambdaHat)*f.matern.euc(distsP, rhoHat, nu)
            diag(C) <- diag(C)+1
            Lt <- chol(C)
            
            CinvY <- backsolve(Lt, backsolve(Lt, y, trans = TRUE))
            tmp <- backsolve(Lt, XP.gp, trans = TRUE)
            XtCinvX <- t(tmp)%*%tmp
            betaHat <- solve(XtCinvX,  crossprod(XP.gp,  CinvY))
            
            resid <- y-XP.gp%*%betaHat
            tmp <- backsolve(Lt, resid, trans = TRUE)
            tau2Hat <- sum(tmp^2)/n
            
            R22 <- f.matern.euc(distsP, rhoHat, nu)
            R12 <- f.matern.euc(dd[(1:m), (m+1):(m+n)], rhoHat, nu)
            C <- R22
            diag(C) <- diag(C)+lambdaHat
            gHat <- X%*%betaHat + R12 %*% solve(C, y - XP.gp%*%betaHat)
            
            sseGP[cnt, it, ] <- c(sum((g-gHat)^2), sum((g-gHat)[inhull == TRUE]^2), sum((g-gHat)[close == TRUE]^2), sum((g-gHat)[close == FALSE & inhull == TRUE]^2))
            mseGP[cnt,  it, ] <- c(mean((g-gHat)^2), mean((g-gHat)[inhull == TRUE]^2), mean((g-gHat)[close == TRUE]^2), mean((g-gHat)[close == FALSE & inhull == TRUE]^2))
            
            gHatGP[cnt, it, ] <- gHat
            lambdaHatGP[cnt, it] <- lambdaHat
            tau2HatGP[cnt, it] <- tau2Hat
            rhoHatGP[cnt, it] <- rhoHat
            betaHatGP[cnt, it, ] <- betaHat
            
          }
          cnt <- cnt + 1
        }
      }
    }
  }
  assign(paste('sseBin', n, sep = ''), sseBin)
  assign(paste('sseTps', n, sep = ''), sseTps)
  assign(paste('sseGP', n, sep = ''), sseGP)
  # I don't have code for saving other output objects here since not used below, but would need to insert this if want to save separate objects for n=100 and n=1000
}
  
### Fig. 5b and 6b

for(n in c(100, 1000)) {

  sseBin <- get(paste('sseBin', n, sep = ''))[ , , 2]  # 2nd element is results for inside convex hull
  sseTps <- get(paste('sseTps', n, sep = ''))[ , , 2]
  sseGP <- get(paste('sseGP', n, sep = ''))[ , , 2]

  if(n == 100) pdf('fig5b.pdf', height = 4, width = 8) else pdf('fig6b.pdf',  height = 4,  width = 8)
  par(mfrow=c(2, 4), omi=c(.4, .6, .45, .4), mgp=c(1.8, .7, 0), mai=rep(.05, 4))
  rhoOffsets <- c(0, 2, 4, 6, 8, 10)
  for(j in c(2, 14, 26, 38)){ # loop over tau values for nu = .5
    index=j
    
    if(j == 2) yaxt <- 's' else yaxt <- 'n'
    # uniform
    plot(rhovals, log2(rowMeans(sseBin[index + rhoOffsets, ]/sseTps[index + rhoOffsets, ])), type='b', ylim=c(-1, 2), ylab='', xlab='', log='x',  xaxt = 'n',  yaxt = yaxt, cex.axis = 1.1, pch=16, cex=.7)
    lines(rhovals, log2(rowMeans(sseGP[index + rhoOffsets, ]/sseTps[index + rhoOffsets, ])), type='b', col='blue',  pch = 16,  cex = .7)
    
    # clustered
    lines(rhovals, log2(rowMeans(sseBin[index + rhoOffsets + 48, ]/sseTps[index + rhoOffsets + 48, ])), type='b', lty=2,  pch = 16,  cex = .7)
    lines(rhovals, log2(rowMeans(sseGP[index + rhoOffsets + 48, ]/sseTps[index + rhoOffsets + 48, ])), type='b', col='blue', lty=2,  pch = 16,  cex = .7)
    abline(h=log2(.9), col='lightgray')
    
    if(j==2 && n == 100)
      legend(.004, 1.4, bty='n', lty=c(1, 2, 1, 2), col=c('black', 'black', 'blue', 'blue'), legend=c('ICAR,  uniform sampling', 'ICAR,  clustered sampling', 'GP,  uniform sampling', 'GP,  clustered sampling'),  cex = .75)
    if(j == 2){
      mtext(expression(nu == 0.5), at=.75, side=2, line=3, outer=TRUE, cex=1)
      mtext(expression(tau^2 == 0.05^2), at=.13, side=3, outer=TRUE, cex=1)
    }
    if(j == 14){
      mtext(expression(tau^2 == 0.15^2), at=.37, side=3, outer=TRUE, cex=1)
    }
    if(j == 26){
      mtext(expression(tau^2 == 0.45^2), at=.63, side=3, outer=TRUE, cex=1)
    }
    if(j == 38){
      mtext(expression(tau^2 == 1.35^2), at=.87, side=3, outer=TRUE, cex=1)
    }
    par(new=TRUE)
    plot(rhovals, rowMeans(sseTps[index + rhoOffsets , ]), pch=16, cex=.7, col='red', xaxt='n', yaxt='n', xlab='', ylab='', ylim=c(1, 11000), log='xy', type='b')
    lines(rhovals, rowMeans(sseTps[index + rhoOffsets + 48, ]), pch=16, cex=.7, col='red', lty = 2, type='b')
    if(j == 38) axis(4, col='red', col.lab='red', col.axis='red',  cex.axis = 1.1)
  }
  
  for(j in c(1, 13, 25, 37)){ # loop over tau values for nu = 2
    index=j
    if(j == 1) yaxt <- 's' else  yaxt <- 'n'
   # uniform
    plot(rhovals, log2(rowMeans(sseBin[index + rhoOffsets, ]/sseTps[index + rhoOffsets, ])), type='b', ylim=c(-1, 2), ylab='',  xlab='', log='x',  cex.axis = 1.1,  yaxt = yaxt, pch=16, cex=.7)
    lines(rhovals, log2(rowMeans(sseGP[index + rhoOffsets, ]/sseTps[index + rhoOffsets, ])), type='b', col='blue',  pch = 16,  cex = .7)
    
    # clustered
    lines(rhovals, log2(rowMeans(sseBin[index + rhoOffsets + 48, ]/sseTps[index + rhoOffsets + 48, ])), type='b', lty=2,  pch = 16,  cex = .7)
    lines(rhovals, log2(rowMeans(sseGP[index + rhoOffsets + 48, ]/sseTps[index + rhoOffsets + 48, ])), type='b', col='blue', lty=2,  pch = 16,  cex = .7)
    abline(h=log2(.9), col='lightgray')
    
    if(j == 1){
      mtext(expression(nu == 2), at=.25, side=2, line=3, outer=TRUE, cex=1)
    }
    par(new=TRUE)
    plot(rhovals, rowMeans(sseTps[index + rhoOffsets, ]), pch=16, cex=.7, col='red', xaxt='n', yaxt='n', xlab='', ylab='', ylim=c(1, 11000), log='xy', type='b')
    lines(rhovals, rowMeans(sseTps[index + rhoOffsets + 48, ]), pch=16, cex=.7, col='red', lty = 2, type='b')
    if(j == 37) axis(4, col='red', col.lab='red', col.axis='red',  cex.axis = 1.1)
  }
  title(xlab = expression(rho),  outer = TRUE,  line = 1.5,  font.lab = 1,  cex.lab = 1.2)
  title(ylab = "log2 of SSE relative to TPS-MRF",  outer = TRUE,  line = 1.45,  font.lab = 1,  cex.lab = 1.2)
  txt <- "(b) simulation results"
  mtext('TPS-MRF SSE',  4,  outer = TRUE,  line = 1.7,  col = "red", cex=.8,  at = .5)
  title(main = txt,  outer = TRUE,  line = 2,  cex.main = 1.6,  font.main = 1)
  
  dev.off()
}

### Fig. 7

library(fields)

m1 <- m2 <- 100
m <- m1 * m2

# generate precision matrices
Qbin <- graphRead('bin', m1, m2)[[1]]
Qtps <- graphRead('tps', m1, m2)[[1]]

locs <- expand.grid(seq(0, 1, len = m1), seq(0, 1, len = m2))

dd <- rdist(locs)

set.seed(0)
n <- 100
filledCells <- sample(1:m, n, replace = TRUE)
PtP <- rep(0, m)
PtP[sort(unique(filledCells))] <- table(filledCells)

Pt <- matrix(0, nr = m, nc = n)
Pt[cbind(filledCells, 1:n)] <- 1
Pts <- as.spam(Pt)

sig2 <- 1
tau2vals <- (c(.05, .15, .45, 1.35))^2
rhovals <- c(.005, .02, .08, .32, 1.28, 2.56)
nuvals <- c(.5, 2)

whichk <- c(1, 2, 2, 2)
whichl <- c(2, 3, 5, 5)
whichmm <- c(2, 2, 4, 2)

# optimal lambda values based on oracle analysis done elsewhere
lambdaBinVals <- exp(c(2.78, -15, 1.38, -15)) 
lambdaTpsVals <- exp(c(9.75, 1.03, 8.14, 3.99)) 

u <- rnorm(m)
upt <- rnorm(n)

# generate surfaces and fits
g <- gHatTps <- gHatBin <- y <- list()
for(p in 1:4){
 
  rho <- rhovals[whichl[p]]
  nu <- nuvals[whichk[p]]
  tau2 <- tau2vals[whichmm[p]]
  
 Sigma <- sig2*f.matern.euc(dd, rho, nu)
  L <- t(chol(Sigma))
  
  lambdaBin <- lambdaBinVals[p]
  lambdaTps <- lambdaTpsVals[p]

  g[[p]] <- matrix(L%*%u, m1, m2)
  y[[p]] <- g[[p]][filledCells]+sqrt(tau2)*upt
  Pty <- Pts%*%y[[p]]
  
  Minv <- lambdaBin*Qbin
  diag(Minv) <- diag(Minv)+PtP
  gHatBin[[p]] <- solve(Minv, Pty)

  Minv <- lambdaTps*Qtps
  diag(Minv) <- diag(Minv)+PtP
  gHatTps[[p]] <- solve(Minv, Pty)

  print((sum((g[[p]]-gHatBin[[p]])^2)))
  print((sum((g[[p]]-gHatTps[[p]])^2)))
  print((sum((g[[p]]-mean(g[[p]]))^2)))
}


pdf('fig7.pdf', width = 8, height = 8)
cexV <- 1.2
par(mfcol = c(4, 4), mai = c(.05, .05, .05, .05), omi = c(0, .2, .2, 0))
atvals <- c(.12, .37, .63, .87)
for(p in c(1, 2, 4, 3)){
  pmap2(y[[p]], locs[filledCells, ], col = tim.colors(32), zlim = c(-4, 4), shrink = 1, xaxt = 'n', yaxt = 'n')
  box()
  mtext(paste('Scenario ', p, sep = ''), at = atvals[p], side = 3, outer = TRUE, cex = cexV)
  if(p == 1){
    mtext('Observations', at = .87, side = 2, outer = TRUE, cex = cexV)
  }
  image(1:m1, 1:m2, g[[p]], col = tim.colors(32), xaxt = 'n', yaxt = 'n', zlim = c(-4, 4))
  box()
  if(p == 1){
    mtext('True surface', at = .63, side = 2, outer = TRUE, cex = cexV)
  }
  image(1:m1, 1:m2, matrix(gHatBin[[p]], m1, m2), col = tim.colors(32), xaxt='n', yaxt = 'n', zlim = c(-4, 4))
  box()
  if(p == 1){
    mtext('ICAR fit', at = .37, side = 2, outer = TRUE, cex = cexV)
  }
  image(1:m1, 1:m2, matrix(gHatTps[[p]], m1, m2), col=tim.colors(32), xaxt = 'n', yaxt = 'n', zlim = c(-4, 4))
  box()
  if(p == 1){
    mtext('TPS-MRF fit', at = .12, side = 2, outer = TRUE, cex = cexV)
  }
}
dev.off()


### oracle analysis for areal data for Fig. 8a

library(fields)
library(spam)

m1 <- m2 <- 100
m <- m1 * m2

# generate precision matrices
Qbin <- graphRead('bin', m1, m2)[[1]]
Qtps <- graphRead('tps', m1, m2)[[1]]


n <- 100

locs <- expand.grid(seq(0, 1, len = m1), seq(0, 1, len = m2))

gridToObsMap <- NULL
lo <- 1;up <- 10
for(k in 1:10){
  gridToObsMap <- c(gridToObsMap, rep(rep(lo:up, each = 10), 10))
  lo <- lo+10; up <- up+10
}

mn <- m/n

P <- matrix(0, nr = n, nc = m)
for(j in 1:n){
  P[j, gridToObsMap == j] <- 1/mn
}
Ps <- as.spam(P)
Pts <- t(Ps)
rm(P)

PtPs <- Pts%*%Ps

dd <- rdist(locs)

tol <- .001

set.seed(0)

sig2 <- 1
tau2vals <- (c(.05, .15, .45, 1.35))^2
rhovals <- c(.005, .02, .08, .32, 1.28, 2.56)
nuvals <- c(.5, 2)

n <- 100

sseGP <- varGP <- biasVarGP <- sseGPArealAssess <- varGPArealAssess <- biasVarGPArealAssess <- array(NA, c(2, length(rhovals), length(tau2vals)))
sseOracle <- varOracle <- biasVarOracle <- sseOracleArealAssess <- varOracleArealAssess <- biasVarOracleArealAssess <- array(NA, c(2, 2, length(rhovals), length(tau2vals)))
lamOracle <- array(NA, c(2, 2, length(rhovals), length(tau2vals)))

# these functions compute 'continuous' results (i.e., at the point locations)
sseFunBin <- function(logL){
  lambda <- exp(logL)
  Sbin <- lambda*Qbin+PtPs
  Mbin <- solve(Sbin, Pts)
  term1bin <- sum(rowSums(Mbin*Mbin))
  MtMbin <- t(Mbin)%*%Mbin
  PtMtMbin <- Pts%*%MtMbin
  lastTermBin <- sum(PtMtMbin*tPSig)
  termsBin <- -2*sum(Mbin*tPSig)+m1*m2*sig2+lastTermBin
  return(termsBin+tau2*term1bin)
}

biasVarFunBin <- function(logL){
  lambda <- exp(logL)
  Sbin <- lambda*Qbin+PtPs
  Mbin <- solve(Sbin, Pts)
  term1bin <- sum(rowSums(Mbin*Mbin))
  MtMbin <- t(Mbin)%*%Mbin
  PtMtMbin <- Pts%*%MtMbin
  lastTermBin <- sum(PtMtMbin*tPSig)
  termsBin <- -2*sum(Mbin*tPSig)+m1*m2*sig2+lastTermBin
  return(c(tau2*term1bin/(termsBin+tau2*term1bin), tau2*term1bin))
}

sseFunTps <- function(logL){
  lambda <- exp(logL)
  Stps <- lambda*Qtps+PtPs
  Mtps <- solve(Stps, Pts)
  term1tps <- sum(rowSums(Mtps*Mtps))
  MtMtps <- t(Mtps)%*%Mtps
  PtMtMtps <- Pts%*%MtMtps
  lastTermTps <- sum(PtMtMtps*tPSig)
  termsTps <- -2*sum(Mtps*tPSig)+m1*m2*sig2+lastTermTps
  return(termsTps+tau2*term1tps)
}

biasVarFunTps <- function(logL){
  lambda <- exp(logL)
  Stps <- lambda*Qtps+PtPs
  Mtps <- solve(Stps, Pts)
  term1tps <- sum(rowSums(Mtps*Mtps))
  MtMtps <- t(Mtps)%*%Mtps
  PtMtMtps <- Pts%*%MtMtps
  lastTermTps <- sum(PtMtMtps*tPSig)
  termsTps <- -2*sum(Mtps*tPSig)+m1*m2*sig2+lastTermTps
  return(c(tau2*term1tps/(termsTps+tau2*term1tps), tau2*term1tps))
}

# these functions compute results evaluated on the grid
sseFunBinArealAssess <- function(logL){
  lambda <- exp(logL)
  Sbin <- lambda*Qbin+PtPs
  Mbin <- solve(Sbin, Pts)
  PMbin <- Ps%*%Mbin
  term1bin <- sum(PMbin*PMbin)
  tmp <- Pts%*%(t(PMbin)%*%PMbin-2*PMbin)
  term2bin <- sum(tmp*tPSig)
  lastTermBin <- sum(tPSig*Pts)
  return(tau2*term1bin+term2bin+lastTermBin)
}

biasVarFunBinArealAssess <- function(logL){
  lambda <- exp(logL)
  Sbin <- lambda*Qbin+PtPs
  Mbin <- solve(Sbin, Pts)
  PMbin <- Ps%*%Mbin
  term1bin <- sum(PMbin*PMbin)
  tmp <- Pts%*%(t(PMbin)%*%PMbin-2*PMbin)
  term2bin <- sum(tmp*tPSig)
  lastTermBin <- sum(tPSig*Pts)
  return(c(tau2*term1bin/(term2bin+lastTermBin+tau2*term1bin), tau2*term1bin))
}

sseFunTpsArealAssess <- function(logL){
  lambda <- exp(logL)
  Stps <- lambda*Qtps+PtPs
  Mtps <- solve(Stps, Pts)
  PMtps <- Ps%*%Mtps
  term1tps <- sum(PMtps*PMtps)
  tmp <- Pts%*%(t(PMtps)%*%PMtps-2*PMtps)
  term2tps <- sum(tmp*tPSig)
  lastTermTps <- sum(tPSig*Pts)
  return(tau2*term1tps+term2tps+lastTermTps)
}

biasVarFunTpsArealAssess <- function(logL){
  lambda <- exp(logL)
  Stps <- lambda*Qtps+PtPs
  Mtps <- solve(Stps, Pts)
  PMtps <- Ps%*%Mtps
  term1tps <- sum(PMtps*PMtps)
  tmp <- Pts%*%(t(PMtps)%*%PMtps-2*PMtps)
  term2tps <- sum(tmp*tPSig)
  lastTermTps <- sum(tPSig*Pts)
  return(c(tau2*term1tps/(term2tps+lastTermTps+tau2*term1tps), tau2*term1tps))
}


### GP oracle results

for(k in 1:2){ # loop over nu=.5, nu=2
  nu <- nuvals[k]
  for(l in 1:length(rhovals)){
    rho <- rhovals[l]
    R <- f.matern.euc(dd, rho, nu) # slow
    RPt <- R%*%Pts # slow - it just adds up var and cov for all grid cells within each obs block
    PRPt <- Ps%*%RPt # quick; dense result

    for(mm in 1:length(tau2vals)){
      tau2 <- tau2vals[mm]
      lambda <- tau2/sig2

      C <- PRPt
      diag(C) <- diag(C)+lambda
      term <- solve(C, PRPt)
      M <- (1/lambda)*(RPt-RPt%*%term)
      
      MtM <- t(M)%*%M
      PtMtM <- Pts%*%MtM

      term1 <- tau2*sum(diag(MtM))
      term2 <- sig2*sum(PtMtM*RPt)-2*sig2*sum(M*RPt)+sig2*m
      sseGP[k, l, mm] <- term1+term2
      varGP[k, l, mm] <- term1
      biasVarGP[k, l, mm] <- term1/(term1+term2)

      PM <- Ps%*%M
      term1 <- tau2*sum(PM*PM)
      tmp <- Pts%*%(t(PM)%*%PM-2*PM)
      term2 <- sig2*(sum(tmp*RPt)+sum(RPt*Pts))
      
      sseGPArealAssess[k, l, mm] <- term1+term2
      varGPArealAssess[k, l, mm] <- term1
      biasVarGPArealAssess[k, l, mm] <- term1/(term1+term2)
      
      print(c(k, l, mm, date()))
    }
  }
}


### MRF oracle results

for(k in 1:2){
  nu <- nuvals[k]
  for(l in 1:length(rhovals)){
    rho <- rhovals[l]

    tPSig <- sig2*f.matern.euc(dd, rho, nu)%*%Pts # ~3.5 min.
    for(mm in 1:length(tau2vals)){
      tau2 <- tau2vals[mm]

      fitBin4 <- nlminb(1, sseFunBin, lower = -15, upper = 15)
       
      fitTps4 <- nlminb(1, sseFunTps,lower = -15,upper = 15)

      lamOracle[1, k, l, mm] <- fitBin4$par
      sseOracle[1, k, l, mm] <- fitBin4$objective

      lamOracle[2, k, l, mm] <- fitTps4$par
      sseOracle[2, k, l, mm] <- fitTps4$objective

      tmp <- biasVarFunBin(fitBin4$par)
      varOracle[1, k, l, mm] <- tmp[2]
      biasVarOracle[1, k, l, mm] <- tmp[1]
      
      tmp <- biasVarFunTps(fitTps4$par)
      varOracle[2, k, l, mm] <- tmp[2]
      biasVarOracle[2, k, l, mm] <- tmp[1]
      
      tmp <- biasVarFunBinArealAssess(fitBin4$par)
      varOracleArealAssess[1, k, l, mm] <- tmp[2]
      biasVarOracleArealAssess[1, k, l, mm] <- tmp[1]
      sseOracleArealAssess[1, k, l, mm] <- tmp[2]/tmp[1]
      
      tmp <- biasVarFunTpsArealAssess(fitTps4$par)
      varOracleArealAssess[2, k, l, mm] <- tmp[2]
      biasVarOracleArealAssess[2, k, l, mm] <- tmp[1]
      sseOracleArealAssess[2, k, l, mm] <- tmp[2]/tmp[1]

      print(c(k, l, mm, date()))
    }
  }
}


### Fig. 8a

type <- 'areal'
pdf('fig8a.pdf', height = 4, width = 8) # or fig6a.pdf or fig8a.pdf
par(mfrow = c(2, 4), mai = rep(.05,  4), mgp = c(1.8, .7, 0), omi = c(.4, .6, .45, .4))
for(k in 1:2){  # loop over nu=1,nu=2
  for(mm in 1:length(tau2vals)){ # loop over tau2 values 
    plotFun(sseOracle, sseGP, sseOracleArealAssess, sseGPArealAssess, k, mm, type = type)
    if(k == 1 && mm == 1)
      mtext(expression(nu == 0.5),at = .77,side = 2,outer = TRUE,cex = 1, line = 3)
    if(k == 2 && mm == 1){
      mtext(expression(nu == 2),at = .27,side = 2,outer = TRUE,cex = 1, line = 3)
    }
    if(k == 1 && mm == 1){
      mtext(expression(tau^2 == 0.05^2),at = .13,side = 3,outer = TRUE,cex = 1)
    }
    if(k == 1 && mm == 2){
      mtext(expression(tau^2 == 0.15^2),at = .37,side = 3,outer = TRUE,cex = 1)
    }
    if(k == 1 && mm == 3){
      mtext(expression(tau^2 == 0.45^2),at=.63,side = 3,outer = TRUE,cex = 1)
    }
    if(k == 1 && mm == 4){
      mtext(expression(tau^2 == 1.35^2),at = .87,side = 3,outer = TRUE,cex = 1)
    }
    if(k == 2 && mm == 4)
      mtext('TPS-MRF SSE', outer = TRUE, side = 4, line = 1.7, col = "red", cex=.8, at = .5) 
    if(k == 1 && mm == 1)
      title(main = "(a) oracle results", outer = TRUE, line = 2, cex.main = 1.6, font.main = 1)
  }
}
dev.off()

### simulations for areal data for Fig. 8b

fitGP <- FALSE # too computationally intensive

library(splancs)
library(fields)
library(spam)

nReps <- 100  

pattern <- 'areal'
n <- 100 

tau2vals <- (c(.05,.15,.45,1.35))^2
rhovals <- c(.005, .02, .08, .32, 1.28, 2.56)

cover <- TRUE

m1 <- m2 <- 100
m <- m1 * m2

# generate precision matrices
Qbin <- graphRead('bin', m1, m2)[[1]]
Qtps <- graphRead('tps', m1, m2)[[1]]

gridLocs <- expand.grid(x = seq(0, 1, len = m1),  y = seq(0, 1, len = m2))
dd <- rdist(gridLocs)

idenM <- as.spam(diag(rep(1, m)))

gridToObsMap <- NULL
lo <- 1;up <- 10
for(k in 1:10){
  gridToObsMap <- c(gridToObsMap, rep(rep(lo:up, each = 10), 10))
  lo <- lo+10; up <- up+10
}

mn <- m/n

P <- matrix(0, nr=n, nc = m)
for(j in 1:n){
  P[j, gridToObsMap  == j] <- 1/mn
}
P <- as.spam(P)
Pt <- t(P)

PtP <- Pt%*%P


X <- as.matrix(cbind(rep(1, m), gridLocs))
PX <- P%*%X
PXt <- t(PX)


# this just sets up the ordering in the initial Cholesky decomposition
tmp <- Qbin + PtP
chBin <- chol.spam(tmp)
tmp <- Qtps + PtP
chTps <- chol.spam(tmp)

tau2HatBin <- tau2HatTps <- lambdaHatBin <- lambdaHatTps <- array(NA, c(48, nReps))
lambdaHatGP <- tau2HatGP <- rhoHatGP <- array(NA, c(48, nReps))
betaHatGP <- array(NA,c(96, nReps, 3))
gSave <- gHatBin <- gHatTps <- array(NA, c(48, nReps, m))
PgHatBin <- PgHatTps <- array(NA, c(48, nReps, n))
sseBin <- sseBinPg <- sseTps <- sseTpsPg <- array(NA, c(48, nReps))

if(cover){
  gSEBin <- gSETps <- gHatBin
  PgSEBin <- PgSETps <- yPrSEBin <- yPrSETps <- PgHatBin
  covBin <- covBinPg <- covBinPr <- covTps <- covTpsPg <- covTpsPr <- array(NA, c(48, nReps))
}
gHatGP <- gHatGPorac <- gHatBin
sseGP <- sseGPPg <- sseGPorac <- sseGPoracPg <- sseBin

sig2 <- 1
tol <- .001
mu <- 0
cTps <- 3
cBin <- 1
       
 
# loop over settings; this will take a long time; in reality was done as multiple jobs
cnt <- 1
for(tauI in 1:length(tau2vals)){ # would be better to have this as the inner loop as Sigma does not change when tau2 changes, but ordering in the output objects matters, so be careful
  tau2 <- tau2vals[tauI]
  for(rhoI in 1:6){
    rho <- rhovals[rhoI]
    
    for(nuI in 1:2){
      if(nuI == 1){
        nu <- 2; wrongNu <- .5
      } else{
        nu <- .5; wrongNu <- 2
      }

      Sigma <- sig2*f.matern.euc(dd, rho, nu) # takes a while
      L <- t(chol(Sigma))
      PSigPt <- P%*%(Sigma%*%Pt)
      rm(Sigma)
      
 
      set.seed(0)

      for(it in 1:nReps){
        
        g <- mu+L%*%rnorm(m)
        Pg <- P%*%g
        y <- rnorm(n, Pg, sqrt(tau2))  
        yPr <- rnorm(n, Pg, sqrt(tau2))  
        gSave[cnt, it, ] <- g
        
       # fitting and prediction
        
        yty <- sum(y^2)
        Pty <- Pt%*%y
        
        # bin fitting
        negLogLik <- function(logL){
          lam <- exp(logL)
          Qplus <- lam*Qbin + PtP
          chBin <- update.spam.chol.NgPeyton(chBin, Qplus)
          
          tmp <- forwardsolve(chBin, Pty)
          return(-0.5*(m-cBin)*logL+0.5*(n-cBin)*log(yty-sum(tmp^2))+sum(log(diag(chBin))))
        }
        
        fit4 <- nlminb(1, negLogLik, lower = -15, upper = 15)

        lambdaHat <- exp(fit4$par)
        
        Qplus <- lambdaHat*Qbin + PtP
        chBin <- update.spam.chol.NgPeyton(chBin, Qplus)
        
        tmp <- forwardsolve(chBin, Pty)
        tau2Hat <- (yty-sum(tmp^2))/(n-cBin)
        
        kappaHatBin <- lambdaHat/tau2Hat
        
        gHat <- backsolve(chBin, forwardsolve(chBin, Pty))
        PgHat <- P%*%gHat
        sseBin[cnt, it] <- sum((g-gHat)^2)
        sseBinPg[cnt, it] <- sum((Pg-PgHat)^2)
        
        if(cover){
          tmp <- forwardsolve(chBin, idenM)
          tmp2 <- colSums(tmp*tmp)
          rm(tmp)
          gSE <- sqrt(tau2Hat*tmp2) 

          covBin[cnt, it] <- mean(g<gHat+1.96*gSE & g>gHat-1.96*gSE)
          
          tmp <- forwardsolve(chBin, Pt)
          tmp2 <- colSums(tmp*tmp)
          rm(tmp)
          PgSE <- sqrt(tau2Hat*tmp2)
          covBinPg[cnt, it] <- mean(Pg<PgHat+1.96*PgSE & Pg>PgHat-1.96*PgSE)
          
          yPrSE <- sqrt(tau2Hat*(tmp2+1))
          covBinPr[cnt, it] <- mean(yPr<PgHat+1.96*yPrSE & yPr>PgHat-1.96*yPrSE)
          
          gSEBin[cnt, it, ] <- gSE
          PgSEBin[cnt, it, ] <- PgSE
          yPrSEBin[cnt, it, ] <- yPrSE
        }

        gHatBin[cnt, it, ] <- gHat
        PgHatBin[cnt, it, ] <- PgHat
        lambdaHatBin[cnt, it] <- lambdaHat
        tau2HatBin[cnt, it] <- tau2Hat
        
        # tps fitting
        negLogLik <- function(logL){
          lam <- exp(logL)
          
          Qplus <- lam*Qtps + PtP
          chTps <- update.spam.chol.NgPeyton(chTps, Qplus)
          
          tmp <- forwardsolve(chTps, Pty)
          return(-0.5*(m-cTps)*logL+0.5*(n-cTps)*log(yty-sum(tmp^2))+sum(log(diag(chTps))))
        }

        fit4 <- nlminb(1, negLogLik, lower = -15, upper = 15)
        
        lambdaHat <- exp(fit4$par)
        
        Qplus <- lambdaHat*Qtps+PtP
        chTps <- update.spam.chol.NgPeyton(chTps, Qplus)
        
        tmp <- forwardsolve(chTps, Pty)
        tau2Hat <- (yty-sum(tmp^2))/(n-cTps)
        
        kappaHatTps <- lambdaHat/tau2Hat
        
        gHat <- backsolve(chTps, forwardsolve(chTps, Pty))
        PgHat <- P%*%gHat
        sseTps[cnt, it] <- sum((g-gHat)^2)
        sseTpsPg[cnt, it] <- sum((Pg-PgHat)^2)
        
        if(cover){
          tmp <- forwardsolve(chTps, idenM)
          tmp2 <- colSums(tmp*tmp)
          rm(tmp)
          gSE <- sqrt(tau2Hat*tmp2)
          covTps[cnt, it] <- mean(g<gHat+1.96*gSE & g>gHat-1.96*gSE)

          tmp <- forwardsolve(chTps, Pt)
          tmp2 <- colSums(tmp*tmp)
          rm(tmp)
          PgSE <- sqrt(tau2Hat*tmp2)
          covTpsPg[cnt, it] <- mean(Pg<PgHat+1.96*PgSE & Pg>PgHat-1.96*PgSE)
          
          yPrSE <- sqrt(tau2Hat*(tmp2+1))
          covTpsPr[cnt, it] <- mean(yPr<PgHat+1.96*yPrSE & yPr>PgHat-1.96*yPrSE)
          
          gSETps[cnt, it, ] <- gSE
          PgSETps[cnt, it, ] <- PgSE
          yPrSETps[cnt, it, ] <- yPrSE
        }
        
        gHatTps[cnt, it, ] <- gHat
        PgHatTps[cnt, it, ] <- PgHat
        lambdaHatTps[cnt, it] <- lambdaHat
        tau2HatTps[cnt, it] <- tau2Hat
        
        # GP with known params; beta assumed 0
        tmp <- PSigPt
        diag(tmp) <- diag(tmp)+tau2
        vec <- solve(tmp, PSigPt%*%y)
        gHat <- (1/tau2)*(L%*%(t(L)%*%(Pty-Pt%*%vec)))
        
        sseGPorac[cnt, it] <- sum((g-gHat)^2)
        gHatGPorac[cnt, it, ] <- gHat
        sseGPoracPg[cnt, it] <- sum((Pg-P%*%gHat)^2)
        
        # GP w/ estimated params
        if(fitGP){
          negLogLik <- function(par){
            lam <- exp(par[1])
            rhoVal <- exp(par[2])
            
            tmp <- f.matern.euc(dd, rhoVal, nu)
            C <- lam*P%*%tmp%*%Pt
            diag(C) <- diag(C)+1
            Lt <- chol(C)
            
            CinvY <- backsolve(Lt, backsolve(Lt, y, trans = TRUE))
            tmp <- backsolve(Lt, PX, trans = TRUE)
            XtCinvX <- t(tmp)%*%tmp
            vec <- y-PX%*%solve(XtCinvX, PXt%*%CinvY)
            CinvVec <- backsolve(Lt, vec, trans = TRUE)
            return(sum(log(diag(Lt)))+(n/2)*log(sum(CinvVec^2)))
          }
          
          fit4 <- nlminb(c(1, log(.25)), negLogLik, lower = c(-15, log(.001)), upper = c(15, log(10)))

          lambdaHat <- exp(fit4$par[1])
          rhoHat <- exp(fit4$par[2])
          
          R <- f.matern.euc(dd, rhoHat, nu)
          C <- PRPt <- lambdaHat*P%*%R%*%Pt
          diag(C) <- diag(C)+1
          Lt <- chol(C)
          CinvY <- backsolve(Lt, backsolve(Lt, y, trans = TRUE))
          tmp <- backsolve(Lt, PX, trans = TRUE)
          XtCinvX <- t(tmp)%*%tmp
          betaHat <- solve(XtCinvX, XPt%*%CinvY)
          resid <- y-PX%*%betaHat
          tmp <- backsolve(Lt, resid, trans = TRUE)
          
          tau2Hat <- sum(tmp^2)/n
          sig2Hat <- lambdaHat*tau2Hat
          
          tmp <- PRPt
          diag(tmp) <- diag(tmp)+(1/lambdaHat)
          
          vec <- solve(tmp, PRPt%*%y)
          gHat <- lambdaHat*R%*%(Pty-Pt%*%vec)
          rm(R)
          
          sseGP[cnt, it] <- sum((g-gHat)^2)
          sseGPPg[cnt, it] <- sum((Pg-P%*%gHat)^2)
          
          gHatGP[cnt, it, ] <- gHat
          lambdaHatGP[cnt, it] <- lambdaHat
          tau2HatGP[cnt, it] <- tau2Hat
          rhoHatGP[cnt, it] <- rhoHat
          betaHatGP[cnt, it, ] <- betaHat
        }
      }
      cnt <- cnt + 1
    }
  }
}
        
        
### Fig. 8b
        
rhovals <- c(.005,  .02,  .08,  .32,  1.28,  2.56)
tau2vals <- (c(.05, .15, .45, 1.35))^2

rhoOffsets <- c(0, 2, 4, 6, 8, 10)

pdf(paste('fig8b.pdf', sep = ''), width = 8, height = 4)
par(mfrow = c(2, 4), omi = c(.4, .6, .45, .4), mgp = c(1.8, .7, 0), mai = rep(.05, 4))
for(j in c(2, 14, 26, 38)){
  index <- j
  if(j == 2) yaxt <- 's' else yaxt <- 'n'
  plot(rhovals, log2(rowMeans(sseBin[index+rhoOffsets, ]/sseTps[index+rhoOffsets, ])), type = 'b', ylim = c(-1, 2), ylab = '', xlab = '', xaxt = 'n',  yaxt = yaxt,  cex.axis = 1.1, log = 'x')
  lines(rhovals, log2(rowMeans(sseBinPg[index+rhoOffsets, ]/sseTpsPg[index+rhoOffsets, ])), type = 'b', lty = 2)
  
  abline(h = log2(.9), lty = 1, col = 'lightgray')
 
  if(j == 2){
    legend(.004, 1.35, bty='n', lty = c(1, 2), legend = c('error on fine grid', 'error on obs. grid'),  cex  =  .9)
    mtext(expression(nu ==0.5), at = .75, side = 2, line = 3, outer = TRUE, cex = 1)
    mtext(expression(tau^2 == 0.05^2), at = .13, side = 3, outer = TRUE, cex = 1)
  }
  if(j == 14){
    mtext(expression(tau^2 == 0.15^2), at = .37, side = 3, outer = TRUE, cex = 1)
  }
  if(j == 26){
    mtext(expression(tau^2 == 0.45^2), at = .63, side = 3, outer = TRUE, cex = 1)
  }
  if(j == 38){
    mtext(expression(tau^2 == 1.35^2), at = .87, side = 3, outer = TRUE, cex = 1)
  }
  par(new = TRUE)
  plot(rhovals, rowMeans(sseTps[index+rhoOffsets, ]), pch = 16, cex = .7, col = 'red', xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', ylim = c(1, 11000), log = 'xy', type = 'b')
  tmp = rowMeans(sseTpsPg[index+rhoOffsets, ])
  tmp[tmp<1] = .7
  lines(rhovals, tmp, pch = 16, cex = .7, col = 'red', lty = 2, type = 'b')
  if(j == 38) axis(4, col = 'red', col.lab = 'red', col.axis = 'red',  cex.axis  =  1.1)
}

for(j in c(1, 13, 25, 37)){
  index = j
  if(j == 1) yaxt <- 's' else  yaxt <- 'n'
  plot(rhovals, log2(rowMeans(sseBin[index+rhoOffsets, ]/sseTps[index+rhoOffsets, ])), type = 'b', ylim = c(-1, 2), log = 'x', ylab = '',  xlab = '',  cex.axis = 1.1,  yaxt = yaxt)
  lines(rhovals, log2(rowMeans(sseBinPg[index+rhoOffsets, ]/sseTpsPg[index+rhoOffsets, ])), type = 'b', lty = 2)
  abline(h = log2(.9), lty = 1, col = 'lightgray')

  if(j == 1)
    mtext(expression(nu == 2), at = .27, side = 2, outer = TRUE, cex = 1, line = 3)
  par(new = TRUE)
  plot(rhovals, rowMeans(sseTps[index+rhoOffsets, ]), pch = 16, cex = .7, col = 'red', xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', ylim = c(1, 11000), log = 'xy', type = 'b')
  tmp = rowMeans(sseTpsPg[index+rhoOffsets, ])
  tmp[tmp<1] = .7
  lines(rhovals, tmp, pch = 16, cex = .7, col = 'red', lty = 2, type = 'b')
  if(j  ==  37) axis(4, col = 'red', col.lab = 'red', col.axis = 'red',  cex.axis  =  1.1)
}
title(xlab  =  expression(rho), outer  =  TRUE, line = 1.5, font.lab = 1, cex.lab = 1.2)
title(ylab = "log2 of SSE relative to TPS-MRF", outer = TRUE, line = 1.45, font.lab = 1, cex.lab = 1.2)
mtext('TPS-MRF SSE', 4, outer = TRUE, line = 1.7, col = "red",cex=.8, at = .5)
title(main = "(b) simulation results", outer = TRUE, line = 2, cex.main = 1.6, font.main = 1)

dev.off()



### Fig. 9

library(geoR)
library(spam)
library(spdep)
library(mgcv)
library(fields)

# state boundary file
shape <- readShapePoly('uscontermprj')

dat <- read.csv('pm.csv')
# grid used in MRF analysis
grid <- read.csv('gridForPM.csv')

m1 <- length(unique(grid$x))
m2 <- length(unique(grid$y))
m <- m1*m2

cBin <- 1
cTps <- 3

load('Qobjects.RData') # Qbin and Qtps pre-generated in spam format for the large grid used in this example


## fitting functions

negLogLikBin <- function(logL){
  lam <- exp(logL)
  Qplus <- lam*Qbin
  diag(Qplus) <- diag(Qplus)+PtP
  chBin <- update.spam.chol.NgPeyton(chBin, Qplus)

  tmp <- forwardsolve(chBin, PtX)
  betaHat <- solve(XtX-t(tmp)%*%tmp, Xty-t(tmp)%*%forwardsolve(chBin, Pty))

  term1 <- sum(log(diag(chBin)))

  res <- ytr-Xtr%*%betaHat
  Ptres <- Pt%*%res
  tmp <- forwardsolve(chBin, Ptres)
  
  return(term1-0.5*(m-cBin)*logL+0.5*(ntr-cBin)*log(sum(res^2)-sum(tmp^2)))
}

negLogLikTps <- function(logL){
 lam <- exp(logL)
  Qplus <- lam*Qtps
  diag(Qplus) <- diag(Qplus)+PtP
  chTps <- update.spam.chol.NgPeyton(chTps, Qplus)

  tmp <- forwardsolve(chTps, PtX)
  betaHat <- solve(XtX-t(tmp)%*%tmp, Xty-t(tmp)%*%forwardsolve(chTps, Pty))

  term1 <- sum(log(diag(chTps)))

  res <- ytr-Xtr%*%betaHat
  Ptres <- Pt%*%res
  tmp <- forwardsolve(chTps, Ptres)
  
  return(term1-0.5*(m-cTps)*logL+0.5*(ntr-cTps)*log(sum(res^2)-sum(tmp^2)))
}


mrffit  <- function(tr, type = 'bin'){
  if(type == 'bin'){
    Q <- Qbin
  } else{
    Q <- Qtps
  }

  if(type == 'bin'){
    fit4 <- nlminb(1, negLogLikBin, lower = -15, upper = 15)
  } else{
    fit4 <- nlminb(1, negLogLikTps, lower = -15, upper = 15)
  }
  lambdaHat <- exp(fit4$par)

  Qplus <- lambdaHat*Q
  diag(Qplus) <- diag(Qplus)+PtP

  if(type == 'bin'){
    chBin <- update.spam.chol.NgPeyton(chBin, Qplus)
    tmp <- forwardsolve(chBin, PtX)
    betaHat <- solve(XtX-t(tmp)%*%tmp, Xty-t(tmp)%*%forwardsolve(chBin, Pty))

    res <- ytr-Xtr%*%betaHat
    Ptres <- Pt%*%res
    gHat <- backsolve(chBin, forwardsolve(chBin, Ptres))
  } else{
    chTps <- update.spam.chol.NgPeyton(chTps, Qplus)
    tmp <- forwardsolve(chTps, PtX)
    betaHat <- solve(XtX-t(tmp)%*%tmp, Xty-t(tmp)%*%forwardsolve(chTps, Pty))

    res <- ytr-Xtr%*%betaHat
    Ptres <- Pt%*%res
    gHat <- backsolve(chTps, forwardsolve(chTps, Ptres))
  }
    
  return(list(gHat, betaHat))
}


## full cross-validation analysis

idenM  <- diag.spam(1, m, m)

dat$gamPred <- dat$krigPredMat <- dat$krigPredExp <- dat$binPred <- dat$tpsPred <- rep(NA, nrow(dat))
dat$gamSE <- dat$krigSEMat <- dat$krigSEExp <- dat$binSE <- dat$tpsSE <- rep(NA, nrow(dat))
dat$gamCov <- dat$krigCovMat <- dat$krigCovExp <- dat$binCov <- dat$tpsCov <- rep(NA, nrow(dat))

for(cv in 1:10){ # values of cvset (10-fold)

  # test and train split
  ts <- dat[dat$cvset == cv, ]
  tr <- dat[dat$cvset != cv, ]

  # GAM fitting
  mod <- gam(pm~LDISTA1+LDISTA2+NLCD_URB+LS25_E10+ELEV_NED+s(x, y, k = 250), data = tr)
  pr <- predict(mod, newdata = ts, se.fit = TRUE)
  dat$gamPred[dat$cvset == cv] <- pr$fit
  se <- sqrt(pr$se.fit^2+mod$sig2)
  dat$gamCov[dat$cvset == cv] <- ts$pm<pr$fit+1.96*se & ts$pm>pr$fit-1.96*se
  dat$gamSE[dat$cvset == cv] <- se
  
  geodts <- as.geodata(ts, coords.col = c(57, 58), data.col = 55, covar.names = c('LDISTA1', 'LDISTA2', 'NLCD_URB', 'LS25_E10', 'ELEV_NED'), covar.col = c('LDISTA1', 'LDISTA2', 'NLCD_URB', 'LS25_E10', 'ELEV_NED'))
  geodtr <- as.geodata(tr, coords.col = c(57, 58), data.col = 55, covar.names = c('LDISTA1', 'LDISTA2', 'NLCD_URB', 'LS25_E10', 'ELEV_NED'), covar.col = c('LDISTA1', 'LDISTA2', 'NLCD_URB', 'LS25_E10', 'ELEV_NED'))
  trendtr <- trend.spatial(~coords+LDISTA1+LDISTA2+NLCD_URB+LS25_E10+ELEV_NED, geodtr)
  trendts <- trend.spatial(~coords+LDISTA1+LDISTA2+NLCD_URB+LS25_E10+ELEV_NED, geodts)

  # kriging fitting
  mod2 <- likfit(geodtr, ini.cov.pars = c(5, 1), fix.psiA = TRUE, fix.psiR = TRUE, nugget = 5, fix.kappa = TRUE, kappa = 2, cov.model = 'matern', trend = trendtr)
  mod2a <- likfit(geodtr, ini.cov.pars = c(.3, .05), fix.psiA = TRUE, fix.psiR = TRUE, nugget = 1, fix.kappa = TRUE, kappa = 2, cov.model = 'matern', trend = trendtr)
  mod2b <- likfit(geodtr, ini.cov.pars = c(.3, 1), fix.psiA = TRUE, fix.psiR = TRUE, nugget = .2, fix.kappa = TRUE, kappa = 2, cov.model = 'matern', trend = trendtr)
  if(mod2a$loglik>mod2$loglik){
    mod2 <- mod2a
  }
  if(mod2b$loglik>mod2$loglik){
    mod2 <- mod2b
  }
  pr <- krige.conv(geodtr, locations = geodts$coords, krige = krige.control(obj.model = mod2, trend.d = trendtr, trend.l = trendts), output = output.control(signal = TRUE))
  dat$krigPredMat[dat$cvset == cv] <- pr$predict
  se <- sqrt(pr$krige.var+mod2$tausq)
  dat$krigCovMat[dat$cvset == cv] <-  ts$pm<pr$predict+1.96*se & ts$pm>pr$predict-1.96*se
  dat$krigSEMat[dat$cvset == cv] <- se
  

  mod2 <- likfit(geodtr, ini.cov.pars = c(5, 1), fix.psiA = TRUE, fix.psiR = TRUE, nugget = 5, fix.kappa = TRUE, kappa = .5, cov.model = 'matern', trend = trendtr)
  mod2a <- likfit(geodtr, ini.cov.pars = c(.3, .05), fix.psiA = TRUE, fix.psiR = TRUE, nugget = 1, fix.kappa = TRUE, kappa = .5, cov.model = 'matern', trend = trendtr)
  mod2b <- likfit(geodtr, ini.cov.pars = c(.3, 1), fix.psiA = TRUE, fix.psiR = TRUE, nugget = .2, fix.kappa = TRUE, kappa = .5, cov.model = 'matern', trend = trendtr)
  if(mod2a$loglik>mod2$loglik){
    mod2 <- mod2a
  }
  if(mod2b$loglik>mod2$loglik){
    mod2 <- mod2b
  }
  pr <- krige.conv(geodtr, locations = geodts$coords, krige = krige.control(obj.model = mod2, trend.d = trendtr, trend.l = trendts), output = output.control(signal = TRUE))
  dat$krigPredExp[dat$cvset == cv] <- pr$predict
  se <- sqrt(pr$krige.var+mod2$tausq)
  dat$krigCovExp[dat$cvset == cv] <-  ts$pm<pr$predict+1.96*se & ts$pm>pr$predict-1.96*se
  dat$krigSEExp[dat$cvset == cv] <- se

  Xts <- as.matrix(ts[, c('LDISTA1', 'LDISTA2', 'NLCD_URB', 'LS25_E10', 'ELEV_NED')])
  Xtr <- as.matrix(tr[, c('LDISTA1', 'LDISTA2', 'NLCD_URB', 'LS25_E10', 'ELEV_NED')])
  ytr <- tr$pm
  ntr <- length(ytr)

  # MRF fitting
  filledCells <- tr$gridNum
  PtP <- rep(0, m)
  PtP[sort(unique(filledCells))] <- table(filledCells)
  Pt <- matrix(0, nr = m, nc = nrow(tr))
  Pt[cbind(filledCells, 1:nrow(tr))] <- 1
  Pt <- as.spam(Pt)

  tmp <- Qbin
  diag(tmp) <- diag(tmp)+PtP
  chBin <- chol.spam(tmp)

  yty <- sum(ytr^2)
  Pty <- Pt%*%ytr

  PtX <- Pt%*%Xtr
  XtX <- t(Xtr)%*%Xtr
  Xty <- t(Xtr)%*%ytr

  out <- mrffit(tr, type = 'bin')
  pr <- c(Xts%*%c(out[[2]])+out[[1]][ts$gridNum])
  dat$binPred[dat$cvset == cv] <- pr

  tmp <- forwardsolve(chBin, Pty)
  tau2Hat <- (yty-sum(tmp^2))/(ntr-cBin)
  tmp <- backsolve(chBin, forwardsolve(chBin, idenM[, ts$gridNum]))
  se <- sqrt(tau2Hat*(diag(tmp[ts$gridNum, 1:ncol(tmp)])+1))
  dat$binCov[dat$cvset == cv] <-  ts$pm<pr+1.96*se & ts$pm>pr-1.96*se
  dat$binSE[dat$cvset == cv] <- se
  
  tmp <- Qtps
  diag(tmp) <- diag(tmp)+PtP
  chTps <- chol.spam(tmp)

  out <- mrffit(tr, type = 'tps')
  pr <- Xts%*%out[[2]]+out[[1]][ts$gridNum]
  dat$tpsPred[dat$cvset == cv] <- pr

  tmp <- forwardsolve(chTps, Pty)
  tau2Hat <- (yty-sum(tmp^2))/(ntr-cTps)
  tmp <- backsolve(chTps, forwardsolve(chTps, idenM[, ts$gridNum]))
  se <- sqrt(tau2Hat*(diag(tmp[ts$gridNum, 1:ncol(tmp)])+1))
  dat$tpsCov[dat$cvset == cv] <-  ts$pm<pr+1.96*se & ts$pm>pr-1.96*se
  dat$tpsSE[dat$cvset == cv] <- se

  print(c(cv, date()))
}

# model comparison
sub <- dat[dat$cvset > 0,  ]

R2 <- function(preds, vals){
    return(c(1-sum((vals-preds)^2)/sum((vals-mean(vals))^2), sqrt(mean((vals-preds)^2))))
}

apply(sub[, c('gamPred', 'krigPredMat', 'krigPredExp', 'binPred', 'tpsPred')], 2, R2, sub$pm)


# fitting MRF models to full dataset for plotting purposes

Xtr <- as.matrix(dat[, c('LDISTA1', 'LDISTA2', 'NLCD_URB', 'LS25_E10', 'ELEV_NED')])
ytr <- dat$pm
ntr <- length(ytr)

filledCells <- dat$gridNum
PtP <- rep(0, m)
PtP[sort(unique(filledCells))] <- table(filledCells)
Pt <- matrix(0, nr = m, nc = nrow(dat))
Pt[cbind(filledCells, 1:nrow(dat))] <- 1
Pt <- as.spam(Pt)

tmp <- Qbin
diag(tmp) <- diag(tmp)+PtP
chBin <- chol.spam(tmp)

tmp <- Qtps
diag(tmp) <- diag(tmp)+PtP
chTps <- chol.spam(tmp)

yty <- sum(ytr^2)
Pty <- Pt%*%ytr

PtX <- Pt%*%Xtr
XtX <- t(Xtr)%*%Xtr
Xty <- t(Xtr)%*%ytr

out1 <- mrffit(dat, type = 'bin')
out2 <- mrffit(dat, type = 'tps')
out1[[1]][grid$dataMask == 0] <- NA
out2[[1]][grid$dataMask == 0] <- NA


xs <- sort(unique(grid$x))
ys <- sort(unique(grid$y))
xloc <- 357000
yloc <- 2950000

pdf('fig9.pdf', height = 1.9*1.5, width = 9*1.5)
par(mfrow = c(1, 3), mgp = c(1.5, .6, 0), mai = c(.05, .05, .05, .3), omi = c(0, 0, 0, .35))
pmap2(dat$pm, cbind(dat$XINKM*1000, dat$YINKM*1000), zlim = c(5, 21), xlim = range(grid$x)+c(60000, -55000), ylim = range(grid$y)+c(50000, -50000), xaxt = 'n', yaxt='n', xlab='', ylab = '', shrink = 1)
plot(shape, add = TRUE, border = 'grey')
box()
text(xloc, yloc, '(a)',  cex = 1.5)
image(xs, ys, matrix(out1[[1]], m1, m2), xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', zlim = c(5, 21), col = tim.colors(32))
plot(shape, add = TRUE, border = 'grey')
box()
text(xloc, yloc, '(b)',  cex = 1.5)
image(xs, ys, matrix(out2[[1]], m1, m2), xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', zlim = c(5, 21), col = tim.colors(32))
plot(shape, add = TRUE, border = 'grey')
box()
text(xloc, yloc, '(c)',  cex  =  1.5)
image.plot(xs, ys, matrix(out2[[1]], m1, m2), xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', zlim = c(5, 21), legend.only = TRUE, smallplot = c(.95, 1, .05, .9),  axis.args = list(cex.axis = 1.35))
dev.off()

### Fig. 10

library(classInt)
library(fields)
library(INLA)
library(spdep)

dataAvailable <- FALSE ### Note the breast cancer dataset is not in the public domain,  so is not included in this archive

# removing data from 'white' data object
# white@data[ ,  c(paste('count', 4:11,  sep = ''),  'O')] <- NA
# save(white,  file = 'LAwithoutData.RData')

load('LAwithoutData.RData')

grid <- read.csv('LAgrid.csv', head = TRUE)
grid <-  grid[order(grid$y,  grid$x),  ]
m <- nrow(grid)

xs <- sort(unique(grid[, 2]))
m1 <- length(xs)
ys <- sort(unique(grid[, 3]))
m2 <- length(ys)

cts <- white@data
nCT <- nrow(cts)

# dealing with overlap of census tracts and latent grid cells

int <- read.csv('lacounty00tr_intgrid.csv', head = TRUE, colClasses = c('factor', rep('numeric', 4)))

P <- matrix(0, nr = nCT, nc = m)

for(i in 1:nCT){
  sub <- int[as.character(int$ctidfp00) == as.character(cts$CTIDFP00[i]), ] # may need as.numeric wrapped around the stuff after ==
  pctOutside <- grid$outsidelapct[grid$gridid%in%sub$gridid]
  tmp <- sub[, 3]*(1-pctOutside/100) # We seem to have excluded intersection with completely ocean pixels,  so this just excludes the ocean portion of a grid cell from the weight  # Malibu CT 800401 is example of partly water pixel
  P[i, (sub$gridid+1)] <- tmp/sum(tmp)
}

# setting up dummy vars for poverty covariate

cts$povcat5[is.na(cts$povcat5)&cts$indpov<.2&cts$indpov>=.1] <- 2

extram <- m1*21
fill <- extram-nCT
fakem <- m+extram
wgts <- c(rep(1, m), rep(0, extram))
x1 <- c(rep(0, m), 1*(cts$povcat5 == 1), rep(0, fill)) # leave out as intercept automatically in MRF - and L. Hund uses high poverty as baseline
x2 <- c(rep(0, m), 1*(cts$povcat5 == 2), rep(0, fill))
x3 <- c(rep(0, m), 1*(cts$povcat5 == 3), rep(0, fill))
x4 <- c(rep(0, m), 1*(cts$povcat5 == 4), rep(0, fill))
x5 <- c(rep(0, m), 1*(cts$povcat5 == 5), rep(0, fill)) 

x10 <- 1*(cts$povcat5 == 1)
x20 <- 1*(cts$povcat5 == 2)
x30 <- 1*(cts$povcat5 == 3)
x40 <- 1*(cts$povcat5 == 4)
x50 <- 1*(cts$povcat5 == 5)

A <- cbind(P, diag(rep(1, nCT)), matrix(0, nr = nCT, nc = fill))
As <- Matrix(A, sparse = TRUE)

# TPS-MRF on latent grid
node <- 1:fakem
formula <-  y ~ x2+x3+x4+x5 + f(node, wgts,  model = "rw2d",  nrow = 99,  ncol = 101+21)
mod <- inla(formula, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic  =  TRUE, mlik  =  TRUE, cpo  =  TRUE))
formulaF <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "rw2d",  nrow = 99,  ncol = 101+21, prior = "flat", param = numeric())
modF <- inla(formulaF, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))
formulaG <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "rw2d",  nrow = 99,  ncol = 101+21, prior = 'gaussian', param = c(0, 1/100))
modG <- inla(formulaG, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic  =  TRUE, mlik = TRUE, cpo = TRUE))
# takes about 10 minutes

# ICAR on latent grid
node <- 1:fakem
formula <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "besag", graph.file = 'graphBin99x122.csv')
mod2 <- inla(formula, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))
formulaF <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "besag", graph.file = 'graphBin99x122.csv', prior = 'flat', param = numeric())
mod2F <- inla(formulaF, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))
formulaG <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "besag", graph.file = 'graphBin99x122.csv', prior = 'gaussian', param = c(0, 1/100))
mod2G <-inla(formulaG, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))

# Lindgren approach (numerical results given in text but not plotted in Fig. 10)
node <- 1:fakem
# mod4 has nu=1
formula <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "matern2d",  nrow = 99,  ncol = 101+21,  nu  =  1)
mod4 <- inla(formula, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))
formulaF <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "matern2d",  nrow = 99,  ncol = 101+21,  nu  =  1,  hyper  =  list(range  =  list(param  =  numeric(),  prior  =  "flat"),  prec = list(param = numeric(),  prior = "flat")))
mod4F <- inla(formulaF, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))

# mod5 has nu=2
formula <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "matern2d",  nrow = 99,  ncol = 101+21,  nu = 2)
mod5 <-inla(formula, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))
formulaF <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "matern2d",  nrow = 99,  ncol = 101+21,  nu = 2,  hyper = list(range = list(param = numeric(),  prior = "flat"),  prec = list(param = numeric(),  prior = "flat")))
mod5F <- inla(formulaF, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))

# standard neighborhood-based ICAR
node <- 1:nCT
formula <- y ~ x2+x3+x4+x5 + f(node,  model = "besag", graph.file = 'graphNb.csv')
mod3 <- inla(formula, data = list(y = cts$O, x2 = x20, x3 = x30, x4 = x40, x5 = x50, node = node), E = cts$E, family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))
formulaF <- y ~ x2+x3+x4+x5 + f(node,  model = "besag", graph.file = 'graphNb.csv', prior = "flat", param = numeric())
mod3F <- inla(formulaF, data = list(y = cts$O, x2 = x20, x3 = x30, x4 = x40, x5 = x50, node = node), E = cts$E, family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))
formulaG <- y ~ x2+x3+x4+x5 + f(node,  model = "besag", graph.file = 'graphNb.csv', prior = "gaussian", param = c(0, 1/100))
mod3G <- inla(formulaG, data = list(y = cts$O, x2 = x20, x3 = x30, x4 = x40, x5 = x50, node = node), E = cts$E, family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE))

############################################################################################
# the following runs are per INLA's suggestion about improved computation

# takes forever - after 2-3 days,  get DIC of 8477.482,  sum(log(cpo)) of -4242.172,  pretty similar to w/o extra control points
node <- 1:fakem
formula <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "rw2d",  nrow = 99,  ncol = 101+21)
mod.cpo <- inla(formula, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic  =  TRUE, mlik = TRUE, cpo = TRUE),  control.inla  =  list(int.strategy = "grid",  diff.logdens = 4,  strategy = "laplace",  npoints = 21))

node <- 1:fakem
formula <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "besag", graph.file = 'graphBin99x122.csv')
mod2.cpo <- inla(formula, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE),  control.inla = list(int.strategy = "grid",  diff.logdens = 4,  strategy = "laplace",  npoints = 21))

# very similar to mod3
node <- 1:nCT
formula <- y ~ x2+x3+x4+x5 + f(node,  model = "besag", graph.file = 'graphNb.csv')
mod3.cpo <- inla(formula, data = list(y = cts$O, x2 = x20, x3 = x30, x4 = x40, x5 = x50, node = node), E = cts$E, family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE),  control.inla = list(int.strategy = "grid",  diff.logdens = 4,  strategy = "laplace",  npoints = 21))

node <- 1:fakem
formula <- y ~ x2+x3+x4+x5 + f(node, wgts,  model = "matern2d",  nrow = 99,  ncol = 101+21,  nu  =  1)
mod4.cpo <- inla(formula, data = list(y = cts$O, x2 = x2, x3 = x3, x4 = x4, x5 = x5, node = node, wgts = wgts), E = cts$E, control.predictor = list(A = As), family = 'Poisson', control.compute = list(dic = TRUE, mlik = TRUE, cpo = TRUE),  control.inla = list(int.strategy = "grid",  diff.logdens = 4,  strategy = "laplace",  npoints = 21))
##################################################################################

cex.main <- 3
legend.shrink <- .7
cex.axis <- 2.5
lwdVal <- 0.25
legend.width <- 2.0
pdf('fig10.pdf',  width  = 21,  height  =  21)
par(mfrow = c(2, 2), mai = c(.05, .1, .15, 1.9), mgp = c(.1, .1, .1))
pal <- temp.colors(12)

if(dataAvailable) { # data are not publicly available so this next plot cannot be created
  tmp <- log(cts$O/(cts$E+.01))
  tmp[tmp > 2] <- 2
  tmp[tmp < -2] <- -2
  q5 <- classIntervals(tmp, style = "fixed", fixedBreaks = seq(-2,  2,  len = 13))
  col <- findColours(q5,  pal)
  
  plot(white[[2]],  col = col, border = NA) 
  image.plot(xs,  ys, matrix(0, 99, 101), zlim = c(-2, 2), xlab = '', ylab = '', xaxt = 'n', yaxt = 'n', bty = 'n',  col  =  temp.colors(12), xlim = c(316000, 460000), ylim = c(3720000, 3855000),  legend.only  =  TRUE,  legend.shrink = legend.shrink,  legend.width = legend.width,  cex = 1.8,  cex.axis = 1.8,  cex.lab = 1.8,  axis.args = list(cex.axis = cex.axis))
} else {
  plot(0, 0,  type = 'n')
  text(0, 0,  "data not publicly available")
}
title("(a) raw log rate ratio",  cex.main = cex.main,  line = -3)

if(dataAvailable)
  gHat <- mod3$summary.random[[1]]$mean else gHat <- scan('pmFitMod3.txt')
pal <- temp.colors(12)
q5 <- classIntervals(gHat, style = "fixed", fixedBreaks=seq(-.3, .3, by=.05))
col <- findColours(q5,  pal)
plot(white,  col = col, border =  NA)
image.plot(xs,  ys, matrix(0, 99, 101), zlim = c(-.3, .3), xlab = '', ylab = '', xaxt = 'n', yaxt = 'n', bty = 'n',  col  =  temp.colors(12), xlim = c(316000, 460000), ylim = c(3720000, 3855000),  legend.only = TRUE,  legend.width = legend.width,  legend.shrink = legend.shrink,  axis.args = list(cex.axis = cex.axis)) 
title("(b) census tract-based ICAR",  cex.main = cex.main,  line = -3)

if(dataAvailable)
  gHat <- mod2$summary.random[[1]]$mean[1:m] else gHat <- scan('pmFitMod2.txt')
gHat[grid$outsidelapct>.5] <- NA
image.plot(xs, ys, matrix(gHat, 99, 101), zlim = c(-.3, .3), xlab = '', ylab = '', xaxt = 'n', yaxt = 'n', bty = 'n',  col  =  temp.colors(12), xlim = c(320000, 442500), ylim = c(3720000, 3875000),  legend.shrink  =  legend.shrink,  legend.width = legend.width,  axis.args = list(cex.axis = cex.axis))
title("(c) ICAR on fine grid",  cex.main = cex.main,  line = -7)

if(dataAvailable)
  gHat <- mod$summary.random[[1]]$mean[1:m] else gHat <- scan('pmFitMod.txt')
gHat[grid$outsidelapct>.5] <- NA
image.plot(xs, ys, matrix(gHat, 99, 101), zlim = c(-.3, .3), xlab = '', ylab = '', xaxt = 'n', yaxt = 'n', bty = 'n',  col  =  temp.colors(12), xlim=c(320000, 442500), ylim=c(3720000, 3875000),  legend.shrink = legend.shrink,  legend.width = legend.width,  axis.args = list(cex.axis = cex.axis))
title("(d) TPS-MRF on fine grid",  cex.main = cex.main,  line = -7)

dev.off()

